from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any

import aiohttp
from awesomeversion import AwesomeVersion

from .consts import (
    DATA_CHANNEL,
    DATA_SOURCE,
    DEFAULT_TIMEOUT,
    HaVersionChannel,
    HaVersionSource,
)
from .exceptions import HaVersionInputException


@dataclass
class HaVersionBase:
    """HaVersion base class."""

    source: HaVersionSource = HaVersionSource.DEFAULT

    channel: HaVersionChannel = HaVersionChannel.DEFAULT

    board: str | None = None
    image: str | None = None

    session: aiohttp.ClientSession | None = field(repr=False, default=None)
    timeout: int = field(repr=False, default=DEFAULT_TIMEOUT)

    _etag: str | None = None

    _version: AwesomeVersion | None = field(repr=False, default=None)
    _version_data: dict[str, Any] = field(repr=False, default_factory=dict)

    def __post_init__(self):
        self.validate_input()

    @property
    def etag(self) -> str | None:
        """Return the etag of the last request if any."""
        return self._etag

    @property
    def version(self) -> AwesomeVersion | None:
        """Return the version."""
        return AwesomeVersion(self._version) if self._version else None

    @property
    def version_data(self) -> dict[str, Any] | None:
        """Return extended version data for supported sources."""
        return {
            DATA_SOURCE: self.source,
            DATA_CHANNEL: self.channel,
            **self._version_data,
        }

    def validate_input(self) -> None:
        """Raise HaVersionInputException if expected input are missing."""
        if self.session is None:
            raise HaVersionInputException("Missing aiohttp.ClientSession")

    async def fetch(self, **kwargs) -> dict[str, Any]:
        """Logic to fetch new version data."""

    def parse(self, data: dict[str, Any]) -> None:
        """Logic to parse new version data."""



import logging
from unittest.mock import patch

import pytest

from pyhaversion import HaVersion as PyHaVersion

logging.basicConfig(level=logging.DEBUG)
pytestmark = pytest.mark.asyncio


async def mocked_fetch(self):
    pass


@pytest.fixture
def HaVersion():
    with (
        patch("pyhaversion.container.HaVersionContainer.fetch", return_value=mocked_fetch),
        patch("pyhaversion.haio.HaVersionHaio.fetch", return_value=mocked_fetch),
        patch("pyhaversion.local.HaVersionLocal.fetch", return_value=mocked_fetch),
        patch("pyhaversion.pypi.HaVersionPypi.fetch", return_value=mocked_fetch),
        patch(
            "pyhaversion.supervisor.HaVersionSupervisor.fetch",
            return_value=mocked_fetch,
        ),
    ):
        yield PyHaVersion



"""pyhaversion package."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from aiohttp.client import ClientTimeout
from awesomeversion import AwesomeVersion

from .base import HaVersionBase
from .consts import DEFAULT_HEADERS, HaVersionChannel
from .exceptions import HaVersionFetchException

URL = "https://registry.hub.docker.com/v2/repositories/homeassistant/home-assistant/tags"


@dataclass
class HaVersionContainer(HaVersionBase):
    """Handle versions for the Container source."""

    async def fetch(self, **kwargs) -> dict[str, Any]:
        """Logic to fetch new version data."""
        request = await self.session.get(
            url=kwargs.get("url", URL),
            headers=DEFAULT_HEADERS,
            timeout=ClientTimeout(total=self.timeout),
        )
        data = await request.json()
        try:
            self.parse(data)
        except KeyError as exception:
            raise HaVersionFetchException(
                "Could not handle response from Docker Hub"
            ) from exception

        if not self.version and (next_url := data.get("next")):
            return await self.fetch(**{"url": next_url})

        return data

    def parse(self, data: dict[str, Any]) -> None:
        """Logic to parse new version data."""
        for image in data["results"]:
            if not (version := image["name"]).startswith("2"):
                continue
            if not len(version.split(".")) >= 3:
                continue

            version = AwesomeVersion(version)
            if version.dev:
                if self.channel == HaVersionChannel.DEV:
                    self._version = version
                    break
            elif version.beta:
                if self.channel == HaVersionChannel.BETA:
                    self._version = version
                    break
            else:
                self._version = version
                break



"""Exceptions for pyhaversion."""


class HaVersionException(Exception):
    """Base pyhaversion exception."""


class HaVersionInputException(HaVersionException):
    """Raised when missing required input."""


class HaVersionFetchException(HaVersionException):
    """Raised there are issues fetching information."""


class HaVersionNotModifiedException(HaVersionFetchException):
    """Raised when the cotent is not modified."""


class HaVersionParseException(HaVersionException):
    """Raised there are issues parsing information."""



"""Const for Tests."""

STABLE_VERSION = "2020.99.9"
BETA_VERSION = "2020.99.9b0"
DEV_VERSION = "2020.99.0.dev19700101"

STABLE_VERSION_BETA_WEEK = "2020.98.9"
BETA_VERSION_BETA_WEEK = "2020.99.9b0"

HEADERS = {"Content-Type": "application/json"}



"""Get the latest Home Assistant version from various sources."""

from __future__ import annotations

import asyncio
from socket import gaierror
from typing import Any

from aiohttp import ClientError, ClientSession
from awesomeversion import AwesomeVersion

from .base import HaVersionBase
from .consts import (
    DEFAULT_BOARD,
    DEFAULT_TIMEOUT,
    LOGGER,
    HaVersionChannel,
    HaVersionSource,
)
from .container import HaVersionContainer
from .exceptions import HaVersionFetchException, HaVersionParseException
from .haio import HaVersionHaio
from .local import HaVersionLocal
from .pypi import HaVersionPypi
from .supervisor import HaVersionSupervisor

_HANDLERS: dict[HaVersionSource, HaVersionBase] = {
    HaVersionSource.CONTAINER: HaVersionContainer,
    HaVersionSource.PYPI: HaVersionPypi,
    HaVersionSource.SUPERVISOR: HaVersionSupervisor,
    HaVersionSource.HAIO: HaVersionHaio,
    HaVersionSource.LOCAL: HaVersionLocal,
    HaVersionSource.DEFAULT: HaVersionLocal,
}


class HaVersion:
    """Haversion client class."""

    def __init__(
        self,
        session: ClientSession | None = None,
        source: HaVersionSource = HaVersionSource.DEFAULT,
        channel: HaVersionChannel = HaVersionChannel.DEFAULT,
        board: str = DEFAULT_BOARD,
        image: str | None = None,
        timeout: int = DEFAULT_TIMEOUT,
    ):
        """Initialize the client."""
        self._handler: HaVersionBase = _HANDLERS[source](
            board=board,
            channel=channel,
            image=image,
            session=session,
            source=source,
            timeout=timeout,
        )

    @property
    def source(self) -> str:
        """Return the source."""
        return self._handler.source

    @property
    def etag(self) -> str | None:
        """Return the etag of the last request if any."""
        return self._handler.etag

    @property
    def version(self) -> AwesomeVersion | None:
        """Return the version."""
        return self._handler.version

    @property
    def version_data(self) -> dict[str, Any]:
        """Return extended version data for supported sources."""
        return self._handler.version_data

    async def get_version(
        self,
        *,
        etag: str | None = None,
    ) -> tuple[AwesomeVersion, dict[str, Any]]:
        """
        Get version update.

        Returns a tupe with version, version_data.
        """
        try:
            data = await self._handler.fetch(etag=etag)

        except asyncio.TimeoutError as exception:
            raise HaVersionFetchException(
                f"Timeout of {self._handler.timeout} seconds was "
                f"reached while fetching version for {self.source}"
            ) from exception

        except (ClientError, gaierror) as exception:
            raise HaVersionFetchException(
                f"Error fetching version information from {self.source} {exception}"
            ) from exception

        try:
            self._handler.parse(data)
        except (KeyError, TypeError, AttributeError) as exception:
            raise HaVersionParseException(
                f"Error parsing version information for {self.source} - {exception}"
            ) from exception

        LOGGER.debug("Version: %s", self.version)
        LOGGER.debug("Version data: %s", self.version_data)
        return self.version, self.version_data



"""Tests for Hassio."""

from unittest.mock import patch

import aiohttp
import pytest

from pyhaversion import (
    HaVersion,
    HaVersionChannel,
    HaVersionInputException,
    HaVersionNotModifiedException,
    HaVersionSource,
)
from tests.common import fixture

from .const import HEADERS, STABLE_VERSION


@pytest.mark.asyncio
async def test_stable_version(aresponses):
    """Test hassio stable."""
    aresponses.add(
        "version.home-assistant.io",
        "/stable.json",
        "get",
        aresponses.Response(text=fixture("supervisor/default", False), status=200, headers=HEADERS),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(session=session, source=HaVersionSource.SUPERVISOR)
        await haversion.get_version()
        assert haversion.version == STABLE_VERSION


@pytest.mark.asyncio
async def test_beta_version(HaVersion):
    """Test hassio beta."""
    with patch(
        "pyhaversion.supervisor.HaVersionSupervisor.fetch",
        return_value=fixture("supervisor/default"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(
                session=session,
                source=HaVersionSource.SUPERVISOR,
                channel=HaVersionChannel.BETA,
                board="test",
                image="test",
            )
            await haversion.get_version()
            assert haversion.version == STABLE_VERSION


@pytest.mark.asyncio
async def test_input_exception(HaVersion):
    """Test input exception."""
    with pytest.raises(HaVersionInputException):
        HaVersion(source=HaVersionSource.SUPERVISOR)


@pytest.mark.asyncio
async def test_etag(aresponses):
    """Test hassio etag."""
    aresponses.add(
        "version.home-assistant.io",
        "/stable.json",
        "get",
        aresponses.Response(
            text=fixture("supervisor/default", False),
            status=200,
            headers={**HEADERS, "Etag": "test"},
        ),
    )
    aresponses.add(
        "version.home-assistant.io",
        "/stable.json",
        "get",
        aresponses.Response(status=304, headers=HEADERS),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(session=session, source=HaVersionSource.SUPERVISOR)
        await haversion.get_version(etag=haversion.etag)
        assert haversion.version == STABLE_VERSION

        with pytest.raises(HaVersionNotModifiedException):
            await haversion.get_version(etag=haversion.etag)



"""Tests for ha.io/version.json."""

import aiohttp
import pytest

from pyhaversion import HaVersion, HaVersionNotModifiedException, HaVersionSource
from pyhaversion.exceptions import HaVersionFetchException
from tests.common import fixture

from .const import HEADERS, STABLE_VERSION


@pytest.mark.asyncio
async def test_haio(aresponses):
    """Test ha.io/version.json stable."""
    aresponses.add(
        "www.home-assistant.io",
        "/version.json",
        "get",
        aresponses.Response(text=fixture("haio/default", False), status=200, headers=HEADERS),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(session=session, source=HaVersionSource.HAIO)
        await haversion.get_version()
        assert haversion.version == STABLE_VERSION


@pytest.mark.asyncio
async def test_etag(aresponses):
    """Test haio etag."""
    aresponses.add(
        "www.home-assistant.io",
        "/version.json",
        "get",
        aresponses.Response(
            text=fixture("haio/default", False),
            status=200,
            headers={**HEADERS, "Etag": "test"},
        ),
    )
    aresponses.add(
        "www.home-assistant.io",
        "/version.json",
        "get",
        aresponses.Response(status=304, headers=HEADERS),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(session=session, source=HaVersionSource.HAIO)
        await haversion.get_version(etag=haversion.etag)
        assert haversion.version == STABLE_VERSION

        with pytest.raises(HaVersionNotModifiedException):
            await haversion.get_version(etag=haversion.etag)


@pytest.mark.asyncio
async def test_bad_json(aresponses):
    """Test bad JSON."""
    aresponses.add(
        "www.home-assistant.io",
        "/version.json",
        "get",
        aresponses.Response(
            text='{"key": "value""}',
            status=200,
            headers=HEADERS,
        ),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(session=session, source=HaVersionSource.HAIO)
        with pytest.raises(
            HaVersionFetchException,
            match="Could not parse JSON from response - Expecting",
        ):
            await haversion.get_version(etag=haversion.etag)



"""Constants for pyhaversion."""

from __future__ import annotations

from enum import StrEnum
import logging

LOGGER: logging.Logger = logging.getLogger(__package__)

DEFAULT_BOARD = "ova"
DEFAULT_IMAGE = "default"
DEFAULT_TIMEOUT = 10
DEFAULT_HEADERS = {
    "User-Agent": "python/pyhaversion",
    "Content-Type": "application/json",
}

DATA_AUDIO = "audio"
DATA_BOARD = "board"
DATA_CHANNEL = "channel"
DATA_CLI = "cli"
DATA_CURRENT_VERSION = "current_version"
DATA_DNS = "dns"
DATA_HASSOS = "hassos"
DATA_HOMEASSISTANT = "homeassistant"
DATA_IMAGE = "image"
DATA_INFO = "info"
DATA_MULTICAST = "multicast"
DATA_OBSERVER = "observer"
DATA_OS = "os"
DATA_RELEASE_DATE = "release_date"
DATA_RELEASE_DESCRIPTION = "release_description"
DATA_RELEASE_NOTES = "release_notes"
DATA_RELEASE_TITLE = "release_title"
DATA_RELEASES = "releases"
DATA_SOURCE = "source"
DATA_SUPERVISOR = "supervisor"
DATA_VERSION = "version"


class HaVersionSource(StrEnum):
    """Valid sources for pyhaversion."""

    CONTAINER = "container"
    HAIO = "haio"
    LOCAL = "local"
    PYPI = "pypi"
    SUPERVISOR = "supervisor"

    DEFAULT = LOCAL


class HaVersionChannel(StrEnum):
    """Valid version channels."""

    BETA = "beta"
    DEV = "dev"
    STABLE = "stable"

    DEFAULT = STABLE



"""pyhaversion package."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from aiohttp.client import ClientTimeout
from aiohttp.hdrs import IF_NONE_MATCH
from awesomeversion import AwesomeVersion

from .base import HaVersionBase
from .consts import (
    DATA_INFO,
    DATA_RELEASES,
    DATA_VERSION,
    DEFAULT_HEADERS,
    HaVersionChannel,
)
from .exceptions import HaVersionNotModifiedException

URL = "https://pypi.org/pypi/homeassistant/json"


@dataclass
class HaVersionPypi(HaVersionBase):
    """Handle versions for the PyPi source."""

    async def fetch(self, **kwargs) -> dict[str, Any]:
        """Logic to fetch new version data."""
        headers = DEFAULT_HEADERS
        if (etag := kwargs.get("etag")) is not None:
            headers[IF_NONE_MATCH] = f'W/"{etag}"'

        request = await self.session.get(
            url=URL,
            headers=headers,
            timeout=ClientTimeout(total=self.timeout),
        )
        self._etag = request.headers.get("etag")

        if request.status == 304:
            raise HaVersionNotModifiedException

        return await request.json()

    def parse(self, data: dict[str, Any]) -> None:
        """Logic to parse new version data."""
        if self.channel != HaVersionChannel.BETA:
            self._version = data.get(DATA_INFO, {}).get(DATA_VERSION)
            return

        self._version = AwesomeVersion(
            sorted(
                [version for version in data.get(DATA_RELEASES, []) if version.startswith("2")],
                reverse=True,
            )[0]
        )



import json
import os


def fixture(filename, asjson=True):
    """Load a fixture."""
    path = os.path.join(os.path.dirname(__file__), "fixtures", f"{filename}.json")
    with open(path, encoding="utf-8") as fptr:
        if asjson:
            return json.loads(fptr.read())
        return fptr.read()



"""Tests for PyPi."""

from unittest.mock import patch

import aiohttp
import pytest

from pyhaversion import (
    HaVersion,
    HaVersionChannel,
    HaVersionInputException,
    HaVersionNotModifiedException,
    HaVersionSource,
)
from tests.common import fixture

from .const import BETA_VERSION, HEADERS, STABLE_VERSION, STABLE_VERSION_BETA_WEEK


@pytest.mark.asyncio
async def test_stable_version(HaVersion):
    """Test pypi stable."""
    with patch(
        "pyhaversion.pypi.HaVersionPypi.fetch",
        return_value=fixture("pypi/default"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(
                session=session,
                source=HaVersionSource.PYPI,
            )
            await haversion.get_version()
            assert haversion.version == STABLE_VERSION
            assert haversion.source == HaVersionSource.PYPI


@pytest.mark.asyncio
async def test_beta_version(HaVersion):
    """Test pypi beta."""
    with patch(
        "pyhaversion.pypi.HaVersionPypi.fetch",
        return_value=fixture("pypi/beta"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(
                session=session,
                source=HaVersionSource.PYPI,
                channel=HaVersionChannel.BETA,
            )
            await haversion.get_version()
            assert haversion.version == BETA_VERSION


@pytest.mark.asyncio
async def test_stable_version_beta_week(aresponses):
    """Test pypi stable during beta week."""
    aresponses.add(
        "pypi.org",
        "/pypi/homeassistant/json",
        "get",
        aresponses.Response(text=fixture("pypi/beta", False), status=200, headers=HEADERS),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(
            session=session,
            source=HaVersionSource.PYPI,
        )
        await haversion.get_version()
        assert haversion.version == STABLE_VERSION_BETA_WEEK


@pytest.mark.asyncio
async def test_input_exception(HaVersion):
    with pytest.raises(HaVersionInputException):
        HaVersion(source=HaVersionSource.PYPI)


@pytest.mark.asyncio
async def test_etag(aresponses):
    """Test pypi etag."""
    aresponses.add(
        "pypi.org",
        "/pypi/homeassistant/json",
        "get",
        aresponses.Response(
            text=fixture("pypi/default", False),
            status=200,
            headers={**HEADERS, "etag": "test"},
        ),
    )
    aresponses.add(
        "pypi.org",
        "/pypi/homeassistant/json",
        "get",
        aresponses.Response(status=304, headers=HEADERS),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(session=session, source=HaVersionSource.PYPI)
        await haversion.get_version(etag=haversion.etag)
        assert haversion.version == STABLE_VERSION

        with pytest.raises(HaVersionNotModifiedException):
            await haversion.get_version(etag=haversion.etag)



"""pyhaversion package."""

from __future__ import annotations

from dataclasses import dataclass
from json import JSONDecodeError
from typing import Any

from aiohttp.client import ClientTimeout
from aiohttp.hdrs import IF_NONE_MATCH

from .base import HaVersionBase
from .consts import (
    DATA_CURRENT_VERSION,
    DATA_RELEASE_DATE,
    DATA_RELEASE_DESCRIPTION,
    DATA_RELEASE_NOTES,
    DATA_RELEASE_TITLE,
    DEFAULT_HEADERS,
)
from .exceptions import HaVersionFetchException, HaVersionNotModifiedException

URL = "https://www.home-assistant.io/version.json"


@dataclass
class HaVersionHaio(HaVersionBase):
    """Handle versions for the home-assistant.io source."""

    async def fetch(self, **kwargs) -> dict[str, Any]:
        """Logic to fetch new version data."""
        headers = DEFAULT_HEADERS
        if (etag := kwargs.get("etag")) is not None:
            headers[IF_NONE_MATCH] = etag

        request = await self.session.get(
            url=URL,
            headers=headers,
            timeout=ClientTimeout(total=self.timeout),
        )
        self._etag = request.headers.get("Etag")

        if request.status == 304:
            raise HaVersionNotModifiedException

        try:
            return await request.json()
        except JSONDecodeError as exception:
            raise HaVersionFetchException(
                f"Could not parse JSON from response - {exception}"
            ) from exception

    def parse(self, data: dict[str, Any]) -> None:
        """Logic to parse new version data."""
        self._version = data.get(DATA_CURRENT_VERSION)
        self._version_data = {
            DATA_RELEASE_DATE: data.get(DATA_RELEASE_DATE),
            DATA_RELEASE_NOTES: data.get(DATA_RELEASE_NOTES),
            DATA_RELEASE_TITLE: data.get(DATA_RELEASE_TITLE),
            DATA_RELEASE_DESCRIPTION: data.get(DATA_RELEASE_DESCRIPTION),
        }



"""Tests for Container."""

from unittest.mock import patch

import aiohttp
import pytest

from pyhaversion import HaVersion, HaVersionException
from pyhaversion.consts import HaVersionChannel, HaVersionSource
from tests.common import fixture

from .const import (
    BETA_VERSION,
    BETA_VERSION_BETA_WEEK,
    DEV_VERSION,
    HEADERS,
    STABLE_VERSION,
    STABLE_VERSION_BETA_WEEK,
)


@pytest.mark.asyncio
async def test_stable_version(HaVersion):
    """Test container stable."""
    with patch(
        "pyhaversion.container.HaVersionContainer.fetch",
        return_value=fixture("container/default"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(session=session, source=HaVersionSource.CONTAINER)
            await haversion.get_version()
            assert haversion.version == STABLE_VERSION


@pytest.mark.asyncio
async def test_beta_version(HaVersion):
    """Test container beta."""
    with patch(
        "pyhaversion.container.HaVersionContainer.fetch",
        return_value=fixture("container/beta_week"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(
                session=session,
                source=HaVersionSource.CONTAINER,
                channel=HaVersionChannel.BETA,
            )
            await haversion.get_version()
            assert haversion.version == BETA_VERSION


@pytest.mark.asyncio
async def test_dev_version(HaVersion):
    """Test container dev."""
    with patch(
        "pyhaversion.container.HaVersionContainer.fetch",
        return_value=fixture("container/default"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(
                session=session,
                source=HaVersionSource.CONTAINER,
                channel=HaVersionChannel.DEV,
            )
            await haversion.get_version()
            assert haversion.version == DEV_VERSION


@pytest.mark.asyncio
async def test_stable_version_beta_week(HaVersion):
    """Test container stable during beta week."""
    with patch(
        "pyhaversion.container.HaVersionContainer.fetch",
        return_value=fixture("container/beta_week"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(
                session=session,
                source=HaVersionSource.CONTAINER,
            )
            await haversion.get_version()
            assert haversion.version == STABLE_VERSION_BETA_WEEK


@pytest.mark.asyncio
async def test_beta_version_beta_week(HaVersion):
    """Test container beta during beta week."""
    with patch(
        "pyhaversion.container.HaVersionContainer.fetch",
        return_value=fixture("container/beta_week"),
    ):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(
                session=session,
                source=HaVersionSource.CONTAINER,
                channel=HaVersionChannel.BETA,
            )
        await haversion.get_version()
        assert haversion.version == BETA_VERSION_BETA_WEEK


@pytest.mark.asyncio
async def test_stable_version_pagination(aresponses):
    """Test container beta during beta week."""
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags",
        "get",
        aresponses.Response(text=fixture("container/page1", False), status=200, headers=HEADERS),
    )
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags/page2",
        "get",
        aresponses.Response(text=fixture("container/page2", False), status=200, headers=HEADERS),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(
            session=session,
            source=HaVersionSource.CONTAINER,
        )
        await haversion.get_version()
        assert haversion.version == STABLE_VERSION


@pytest.mark.asyncio
async def test_beta_version_pagination(aresponses):
    """Test container beta during beta week."""
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags",
        "get",
        aresponses.Response(
            text=fixture("container/beta_week_page1", False),
            status=200,
            headers=HEADERS,
        ),
    )
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags/page2",
        "get",
        aresponses.Response(
            text=fixture("container/beta_week_page2", False),
            status=200,
            headers=HEADERS,
        ),
    )
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(
            session=session,
            source=HaVersionSource.CONTAINER,
            channel=HaVersionChannel.BETA,
        )
        await haversion.get_version()
        assert haversion.version == BETA_VERSION


@pytest.mark.asyncio
async def test_stable_version_beta_week_pagination(aresponses):
    """Test container beta during beta week."""
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags",
        "get",
        aresponses.Response(
            text=fixture("container/beta_week_page1", False),
            status=200,
            headers=HEADERS,
        ),
    )
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags/page2",
        "get",
        aresponses.Response(
            text=fixture("container/beta_week_page2", False),
            status=200,
            headers=HEADERS,
        ),
    )

    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(
            session=session,
            source=HaVersionSource.CONTAINER,
        )
        await haversion.get_version()
        assert haversion.version == STABLE_VERSION_BETA_WEEK


@pytest.mark.asyncio
async def test_beta_version_beta_week_pagination(aresponses):
    """Test container beta during beta week."""
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags",
        "get",
        aresponses.Response(
            text=fixture("container/beta_week_page1", False),
            status=200,
            headers=HEADERS,
        ),
    )
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags/page2",
        "get",
        aresponses.Response(
            text=fixture("container/beta_week_page2", False),
            status=200,
            headers=HEADERS,
        ),
    )

    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(
            session=session,
            source=HaVersionSource.CONTAINER,
            channel=HaVersionChannel.BETA,
        )
        await haversion.get_version()
        assert haversion.version == BETA_VERSION_BETA_WEEK


@pytest.mark.asyncio
async def test_keyerror(aresponses):
    """Test container KeyError."""
    aresponses.add(
        "registry.hub.docker.com",
        "/v2/repositories/homeassistant/home-assistant/tags",
        "get",
        aresponses.Response(
            text="{}",
            status=200,
            headers=HEADERS,
        ),
    )

    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(
            session=session,
            source=HaVersionSource.CONTAINER,
            channel=HaVersionChannel.BETA,
        )
        with pytest.raises(HaVersionException):
            await haversion.get_version()
            assert haversion.version is None

        haversion._handler._version = "1.2.3"

        with pytest.raises(HaVersionException):
            await haversion.get_version()
            assert haversion.version == "1.2.3"



"""Tests for ha.io/version.json."""

from unittest.mock import patch

import aiohttp
import pytest

from pyhaversion import HaVersion
from pyhaversion.consts import HaVersionSource

from .const import STABLE_VERSION


@pytest.mark.asyncio
async def test_local():
    """Test ha.io/version.json stable."""
    async with aiohttp.ClientSession() as session:
        haversion = HaVersion(session=session, source=HaVersionSource.LOCAL)
        await haversion.get_version()
        assert haversion.version is None

    with patch("pyhaversion.local.localversion", STABLE_VERSION):
        async with aiohttp.ClientSession() as session:
            haversion = HaVersion(session=session, source=HaVersionSource.LOCAL)
            await haversion.get_version()
            assert haversion.version == STABLE_VERSION



"""HaVersionLocal class."""

from __future__ import annotations

from dataclasses import dataclass

from .base import HaVersionBase

try:
    from homeassistant.const import __version__ as localversion
except (ModuleNotFoundError, ImportError):
    localversion = None


@dataclass
class HaVersionLocal(HaVersionBase):
    """HaVersionLocal class."""

    def __post_init__(self):
        self._version = localversion
        super().__post_init__()

    def validate_input(self) -> None:
        """Raise HaVersionInputException if expected input are missing."""

    @property
    def version_data(self) -> None:
        """Return extended version data."""
        return {}



"""Example usage of pyhaversion."""

import asyncio

import aiohttp

from pyhaversion import HaVersion
from pyhaversion.consts import HaVersionChannel, HaVersionSource


async def example():
    """Example usage of pyhaversion."""
    async with aiohttp.ClientSession() as session:
        sources = [
            HaVersionSource.CONTAINER,
            HaVersionSource.SUPERVISOR,
            HaVersionSource.HAIO,
            HaVersionSource.PYPI,
        ]
        for source in sources:
            version, data = await HaVersion(
                session=session,
                source=source,
                board="generic-x86-64",
                channel=HaVersionChannel.DEFAULT,
            ).get_version()
            print(source)
            print("Version:", version)
            print("Version data:", data)
            print()


asyncio.run(example())



"""Tests for base"""

import asyncio
from socket import gaierror
from unittest.mock import patch

from aiohttp import ClientError
import pytest

from pyhaversion import (
    HaVersion,
    HaVersionFetchException,
    HaVersionNotModifiedException,
    HaVersionParseException,
)


@pytest.mark.asyncio
async def test_timeout_exception():
    """Test timeout exception."""

    async def mocked_fetch_TimeoutError(*_, **__):
        """mocked"""
        raise asyncio.TimeoutError

    with patch("pyhaversion.local.HaVersionLocal.fetch", mocked_fetch_TimeoutError):
        haversion = HaVersion()
        with pytest.raises(HaVersionFetchException):
            await haversion.get_version()


@pytest.mark.asyncio
async def test_fetch_exception():
    """Test fetch exception."""
    haversion = HaVersion()

    async def mocked_fetch_gaierror(*_, **__):
        """mocked"""
        raise gaierror

    async def mocked_fetch_ClientError(*_, **__):
        """mocked"""
        raise ClientError

    with patch("pyhaversion.local.HaVersionLocal.fetch", mocked_fetch_gaierror):
        with pytest.raises(HaVersionFetchException):
            await haversion.get_version()

    with patch("pyhaversion.local.HaVersionLocal.fetch", mocked_fetch_ClientError):
        with pytest.raises(HaVersionFetchException):
            await haversion.get_version()


@pytest.mark.asyncio
async def test_parse_exception():
    """Test parse exception."""
    haversion = HaVersion()

    async def mocked_fetch(*_, **__):
        """mocked"""
        pass

    def mocked_parse_KeyError(*_):
        """mocked"""
        raise KeyError

    def mocked_parse_TypeError(*_):
        """mocked"""
        raise TypeError

    with patch("pyhaversion.local.HaVersionLocal.fetch", mocked_fetch):
        with patch("pyhaversion.local.HaVersionLocal.parse", mocked_parse_KeyError):
            with pytest.raises(HaVersionParseException):
                await haversion.get_version()

        with patch("pyhaversion.local.HaVersionLocal.parse", mocked_parse_TypeError):
            with pytest.raises(HaVersionParseException):
                await haversion.get_version()


@pytest.mark.asyncio
async def test_not_modified_exception():
    """Test not_modified exception."""

    async def mocked_fetch_not_modified(*_, **__):
        """mocked"""
        raise HaVersionNotModifiedException

    with patch("pyhaversion.local.HaVersionLocal.fetch", mocked_fetch_not_modified):
        haversion = HaVersion()
        with pytest.raises(HaVersionNotModifiedException):
            await haversion.get_version()



"""pyhaversion package."""

from .consts import HaVersionChannel, HaVersionSource
from .exceptions import (
    HaVersionException,
    HaVersionFetchException,
    HaVersionInputException,
    HaVersionNotModifiedException,
    HaVersionParseException,
)
from .version import HaVersion

__all__ = [
    "HaVersion",
    "HaVersionChannel",
    "HaVersionSource",
    "HaVersionException",
    "HaVersionFetchException",
    "HaVersionInputException",
    "HaVersionNotModifiedException",
    "HaVersionParseException",
]



"""pyhaversion package."""

from __future__ import annotations

from dataclasses import dataclass
from typing import Any

from aiohttp.client import ClientTimeout
from aiohttp.hdrs import IF_NONE_MATCH

from .base import HaVersionBase
from .consts import (
    DATA_AUDIO,
    DATA_BOARD,
    DATA_CLI,
    DATA_DNS,
    DATA_HASSOS,
    DATA_HOMEASSISTANT,
    DATA_IMAGE,
    DATA_MULTICAST,
    DATA_OBSERVER,
    DATA_OS,
    DATA_SUPERVISOR,
    DEFAULT_BOARD,
    DEFAULT_HEADERS,
    DEFAULT_IMAGE,
    LOGGER,
)
from .exceptions import HaVersionNotModifiedException

URL = "https://version.home-assistant.io/{channel}.json"


@dataclass
class HaVersionSupervisor(HaVersionBase):
    """Handle versions for the Supervisor source."""

    def validate_input(self) -> None:
        """Raise HaVersionInputException if expected input are missing."""
        super().validate_input()
        if self.image is None:
            self.image = DEFAULT_IMAGE

    async def fetch(self, **kwargs) -> dict[str, Any]:
        """Logic to fetch new version data."""
        headers = DEFAULT_HEADERS
        if (etag := kwargs.get("etag")) is not None:
            headers[IF_NONE_MATCH] = etag

        request = await self.session.get(
            url=URL.format(channel=self.channel),
            headers=headers,
            timeout=ClientTimeout(total=self.timeout),
        )
        self._etag = request.headers.get("Etag")

        if request.status == 304:
            raise HaVersionNotModifiedException

        return await request.json()

    def parse(self, data: dict[str, Any]) -> None:
        """Logic to parse new version data."""
        if self.image != DEFAULT_IMAGE and self.image not in data.get(DATA_HOMEASSISTANT, {}):
            LOGGER.warning("Image '%s' not found, using default '%s'", self.image, DEFAULT_IMAGE)
            self.image = DEFAULT_IMAGE
        self._version = data.get(DATA_HOMEASSISTANT, {}).get(self.image)
        if self.board != DEFAULT_BOARD and self.board not in data.get(DATA_HASSOS, {}):
            LOGGER.warning("Board '%s' not found, using default '%s'", self.board, DEFAULT_BOARD)
            self.board = DEFAULT_BOARD
        self._version_data = {
            DATA_AUDIO: data.get(DATA_AUDIO),
            DATA_BOARD: self.board,
            DATA_CLI: data.get(DATA_CLI),
            DATA_DNS: data.get(DATA_DNS),
            DATA_OS: data.get(DATA_HASSOS, {}).get(self.board),
            DATA_IMAGE: self.image,
            DATA_MULTICAST: data.get(DATA_MULTICAST),
            DATA_OBSERVER: data.get(DATA_OBSERVER),
            DATA_SUPERVISOR: data.get(DATA_SUPERVISOR),
        }
